@@ -52,82 +52,157 @@
 #define NUM_ATTEMPT 20
 #define INT_PREC 0.1
 
-static double change_of_basis_monocli[18][3][3] = {{{ 1, 0, 0 },
-                                                    { 0, 1, 0 },
-                                                    { 0, 0, 1 }},
-                                                   {{ 0, 0, 1 },
-                                                    { 0,-1, 0 },
-                                                    { 1, 0, 0 }},
-                                                   {{ 0, 0, 1 },
-                                                    { 1, 0, 0 },
-                                                    { 0, 1, 0 }},
-                                                   {{ 1, 0, 0 },
-                                                    { 0, 0, 1 },
-                                                    { 0,-1, 0 }},
-                                                   {{ 0, 1, 0 },
-                                                    { 0, 0, 1 },
-                                                    { 1, 0, 0 }},
-                                                   {{ 0,-1, 0 },
-                                                    { 1, 0, 0 },
-                                                    { 0, 0, 1 }},
-                                                   {{-1, 0, 1 },
-                                                    { 0, 1, 0 },
-                                                    {-1, 0, 0 }},
-                                                   {{ 1, 0,-1 },
-                                                    { 0,-1, 0 },
-                                                    { 0, 0,-1 }},
-                                                   {{ 0, 1,-1 },
-                                                    { 1, 0, 0 },
-                                                    { 0, 0,-1 }},
-                                                   {{-1,-1, 0 },
-                                                    { 0, 0, 1 },
-                                                    {-1, 0, 0 }},
-                                                   {{ 1,-1, 0 },
-                                                    { 0, 0, 1 },
-                                                    { 0,-1, 0 }},
-                                                   {{ 0, 1, 1 },
-                                                    { 1, 0, 0 },
-                                                    { 0, 1, 0 }},
-                                                   {{ 0, 0,-1 },
-                                                    { 0, 1, 0 },
-                                                    { 1, 0,-1 }},
-                                                   {{-1, 0, 0 },
-                                                    { 0,-1, 0 },
-                                                    {-1, 0, 1 }},
-                                                   {{ 0,-1, 0 },
-                                                    { 1, 0, 0 },
-                                                    { 0,-1, 1 }},
-                                                   {{ 0, 1, 0 },
-                                                    { 0, 0, 1 },
-                                                    { 1, 1, 0 }},
-                                                   {{-1, 0, 0 },
-                                                    { 0, 0, 1 },
-                                                    {-1, 1, 0 }},
-                                                   {{ 0, 0,-1 },
-                                                    { 1, 0, 0 },
-                                                    { 0,-1,-1 }}};
+static double change_of_basis_monocli[36][3][3] = {
+  {{ 1, 0, 0 }, /* b  first turn; two axes are flipped in second turn */
+   { 0, 1, 0 },
+   { 0, 0, 1 }},
+  {{ 0, 0, 1 }, /* b */
+   { 0,-1, 0 },
+   { 1, 0, 0 }},
+  {{ 0, 0, 1 }, /* a */
+   { 1, 0, 0 },
+   { 0, 1, 0 }},
+  {{ 1, 0, 0 }, /* c */
+   { 0, 0, 1 },
+   { 0,-1, 0 }},
+  {{ 0, 1, 0 }, /* c */
+   { 0, 0, 1 },
+   { 1, 0, 0 }},
+  {{ 0,-1, 0 }, /* a */
+   { 1, 0, 0 },
+   { 0, 0, 1 }},
+  {{-1, 0, 1 }, /* b */
+   { 0, 1, 0 },
+   {-1, 0, 0 }},
+  {{ 1, 0,-1 }, /* b */
+   { 0,-1, 0 },
+   { 0, 0,-1 }},
+  {{ 0, 1,-1 }, /* a */
+   { 1, 0, 0 },
+   { 0, 0,-1 }},
+  {{-1,-1, 0 }, /* c */
+   { 0, 0, 1 },
+   {-1, 0, 0 }},
+  {{ 1,-1, 0 }, /* c */
+   { 0, 0, 1 },
+   { 0,-1, 0 }},
+  {{ 0, 1, 1 }, /* a */
+   { 1, 0, 0 },
+   { 0, 1, 0 }},
+  {{ 0, 0,-1 }, /* b */
+   { 0, 1, 0 },
+   { 1, 0,-1 }},
+  {{-1, 0, 0 }, /* b */
+   { 0,-1, 0 },
+   {-1, 0, 1 }},
+  {{ 0,-1, 0 }, /* a */
+   { 1, 0, 0 },
+   { 0,-1, 1 }},
+  {{ 0, 1, 0 }, /* c */
+   { 0, 0, 1 },
+   { 1, 1, 0 }},
+  {{-1, 0, 0 }, /* c */
+   { 0, 0, 1 },
+   {-1, 1, 0 }},
+  {{ 0, 0,-1 }, /* a */
+   { 1, 0, 0 },
+   { 0,-1,-1 }},
+  {{ 1, 0, 0 }, /* b  two axes are flipped to look for non-acute axes */
+   { 0,-1, 0 },
+   { 0, 0,-1 }},
+  {{ 0, 0,-1 }, /* b */
+   { 0, 1, 0 },
+   { 1, 0, 0 }},
+  {{ 0, 0, 1 }, /* a */
+   {-1, 0, 0 },
+   { 0,-1, 0 }},
+  {{-1, 0, 0 }, /* c */
+   { 0, 0,-1 },
+   { 0,-1, 0 }},
+  {{ 0, 1, 0 }, /* c */
+   { 0, 0,-1 },
+   {-1, 0, 0 }},
+  {{ 0, 1, 0 }, /* a */
+   {-1, 0, 0 },
+   { 0, 0, 1 }},
+  {{-1, 0,-1 }, /* b */
+   { 0,-1, 0 },
+   {-1, 0, 0 }},
+  {{ 1, 0, 1 }, /* b */
+   { 0, 1, 0 },
+   { 0, 0, 1 }},
+  {{ 0,-1,-1 }, /* a */
+   {-1, 0, 0 },
+   { 0, 0,-1 }},
+  {{ 1,-1, 0 }, /* c */
+   { 0, 0,-1 },
+   { 1, 0, 0 }},
+  {{-1,-1, 0 }, /* c */
+   { 0, 0,-1 },
+   { 0,-1, 0 }},
+  {{ 0,-1, 1 }, /* a */
+   {-1, 0, 0 },
+   { 0,-1, 0 }},
+  {{ 0, 0, 1 }, /* b */
+   { 0,-1, 0 },
+   { 1, 0, 1 }},
+  {{-1, 0, 0 }, /* b */
+   { 0, 1, 0 },
+   {-1, 0,-1 }},
+  {{ 0, 1, 0 }, /* a */
+   {-1, 0, 0 },
+   { 0, 1, 1 }},
+  {{ 0, 1, 0 }, /* c */
+   { 0, 0,-1 },
+   {-1, 1, 0 }},
+  {{ 1, 0, 0 }, /* c */
+   { 0, 0,-1 },
+   { 1, 1, 0 }},
+  {{ 0, 0,-1 }, /* a */
+   {-1, 0, 0 },
+   { 0, 1,-1 }}};
 
-static Centering change_of_centering_monocli[18] = {C_FACE,
-                                                    A_FACE,
-                                                    B_FACE,
-                                                    B_FACE,
-                                                    A_FACE,
-                                                    C_FACE,
-                                                    BASE,
-                                                    BASE,
-                                                    BASE,
-                                                    BASE,
-                                                    BASE,
-                                                    BASE,
-                                                    A_FACE,
-                                                    C_FACE,
-                                                    C_FACE,
-                                                    A_FACE,
-                                                    B_FACE,
-                                                    B_FACE};
+static Centering change_of_centering_monocli[36] = {
+  C_FACE, /* first turn */
+  A_FACE,
+  B_FACE,
+  B_FACE,
+  A_FACE,
+  C_FACE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  A_FACE,
+  C_FACE,
+  C_FACE,
+  A_FACE,
+  B_FACE,
+  B_FACE,
+  C_FACE, /* second turn */
+  A_FACE,
+  B_FACE,
+  B_FACE,
+  A_FACE,
+  C_FACE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  A_FACE,
+  C_FACE,
+  C_FACE,
+  A_FACE,
+  B_FACE,
+  B_FACE};
 
-static int change_of_unique_axis_monocli[18] =
-  {1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0};
+static int change_of_unique_axis_monocli[36] = {
+  1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0,
+  1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0};
 
 static double change_of_basis_ortho[6][3][3] = {{{ 1, 0, 0 },
                                                  { 0, 1, 0 },
@@ -227,11 +302,6 @@ static double F_mat[3][3] = {{    0, 1./2, 1./2 },
                              { 1./2,    0, 1./2 },
                              { 1./2, 1./2,    0 }};
 
-static Spacegroup search_spacegroup(const Cell * primitive,
-                                    const int candidates[],
-                                    const int num_candidates,
-                                    const double symprec,
-                                    const double angle_tolerance);
 static Spacegroup search_spacegroup_with_symmetry(const Cell * primitive,
                                                   const int candidates[],
                                                   const int num_candidates,
@@ -297,58 +367,58 @@ static Centering get_centering(double correction_mat[3][3],
                                SPGCONST int transform_mat[3][3],
                                const Laue laue);
 static Centering get_base_center(SPGCONST int transform_mat[3][3]);
+static int get_centering_shifts(double shift[3][3],
+                                const Centering centering);
 
 
-/* NULL is returned if failed */
-Primitive * spa_get_spacegroup(Spacegroup * spacegroup,
-                               const Cell * cell,
-                               const double symprec,
-                               const double angle_tolerance)
+/* Return spacegroup.number = 0 if failed */
+Spacegroup spa_search_spacegroup(const Cell * primitive,
+                                 const int hall_number,
+                                 const double symprec,
+                                 const double angle_tolerance)
 {
-  int attempt;
-  double tolerance;
-  Primitive *primitive;
+  Spacegroup spacegroup;
+  Symmetry *symmetry;
+  int candidate[1];
 
-  debug_print("spa_get_spacegroup (tolerance = %f):\n", symprec);
+  debug_print("search_spacegroup (tolerance = %f):\n", symprec);
 
-  primitive = NULL;
+  symmetry = NULL;
+  spacegroup.number = 0;
 
-  tolerance = symprec;
-
-  for (attempt = 0; attempt < NUM_ATTEMPT; attempt++) {
-    if ((primitive = prm_get_primitive(cell, tolerance, angle_tolerance)) ==
-        NULL) {
-      goto cont;
-    }
-
-    *spacegroup = search_spacegroup(primitive->cell,
-                                    spacegroup_to_hall_number,
-                                    230,
-                                    primitive->tolerance,
-                                    primitive->angle_tolerance);
-    if (spacegroup->number > 0) {
-      break;
-    } else {
-      prm_free_primitive(primitive);
-      primitive = NULL;
-    }
-
-  cont:
-    warning_print("spglib: Attempt %d tolerance = %f failed.",
-                  attempt, tolerance);
-    warning_print(" (line %d, %s).\n", __LINE__, __FILE__);
-
-    tolerance *= REDUCE_RATE;
+  if ((symmetry = sym_get_operation(primitive, symprec, angle_tolerance)) ==
+      NULL) {
+    goto ret;
   }
 
-  if (primitive == NULL) {
-    warning_print("spglib: Space group could not be found ");
-    warning_print("(line %d, %s).\n", __LINE__, __FILE__);
+  if (hall_number > 0) {
+    candidate[0] = hall_number;
   }
 
-  return primitive;
+  if (hall_number) {
+    spacegroup = search_spacegroup_with_symmetry(primitive,
+                                                 candidate,
+                                                 1,
+                                                 symmetry,
+                                                 symprec,
+                                                 angle_tolerance);
+  } else {
+    spacegroup = search_spacegroup_with_symmetry(primitive,
+                                                 spacegroup_to_hall_number,
+                                                 230,
+                                                 symmetry,
+                                                 symprec,
+                                                 angle_tolerance);
+  }
+
+  sym_free_symmetry(symmetry);
+  symmetry = NULL;
+
+ ret:
+  return spacegroup;
 }
 
+
 Spacegroup spa_search_spacegroup_with_symmetry(const Symmetry *symmetry,
                                                const double symprec)
 {
@@ -370,53 +440,21 @@ Spacegroup spa_search_spacegroup_with_symmetry(const Symmetry *symmetry,
   return spacegroup;
 }
 
-/* Return spacegroup.number = 0 if failed */
-Spacegroup spa_get_spacegroup_with_hall_number(const Primitive * primitive,
-                                               const int hall_number)
-{
-  int num_candidates;
-  int candidate[1];
-  Spacegroup spacegroup;
-
-  spacegroup.number = 0;
-
-  if (hall_number < 1 || hall_number > 530) {
-    goto err;
-  }
-
-  num_candidates = 1;
-  candidate[0] = hall_number;
-  spacegroup = search_spacegroup(primitive->cell,
-                                 candidate,
-                                 num_candidates,
-                                 primitive->tolerance,
-                                 primitive->angle_tolerance);
-  if (spacegroup.number > 0) {
-    goto ret;
-  }
-
- err:
-  warning_print("spglib: Space group with the input choice could not be found ");
-  warning_print("(line %d, %s).\n", __LINE__, __FILE__);
-
- ret:
-  return spacegroup;
-}
-
 /* Return NULL if failed */
-Cell * spa_transform_to_primitive(const Cell * cell,
+Cell * spa_transform_to_primitive(int * mapping_table,
+                                  const Cell * cell,
                                   SPGCONST double trans_mat[3][3],
                                   const Centering centering,
                                   const double symprec)
 {
-  int * mapping_table;
   double tmat[3][3], tmat_inv[3][3], prim_lat[3][3];
   Cell * primitive;
 
-  mapping_table = NULL;
   primitive = NULL;
 
-  mat_inverse_matrix_d3(tmat_inv, trans_mat, 0);
+  if (!mat_inverse_matrix_d3(tmat_inv, trans_mat, symprec)) {
+    goto err;
+  }
 
   switch (centering) {
   case PRIMITIVE:
@@ -441,16 +479,12 @@ Cell * spa_transform_to_primitive(const Cell * cell,
     goto err;
   }
 
-  if ((mapping_table = (int*) malloc(sizeof(int) * cell->size)) == NULL) {
-    warning_print("spglib: Memory could not be allocated ");
-    goto err;
-  }
-
   mat_multiply_matrix_d3(prim_lat, cell->lattice, tmat);
-  primitive = cel_trim_cell(mapping_table, prim_lat, cell, symprec);
-
-  free(mapping_table);
-  mapping_table = NULL;
+  if ((primitive = cel_trim_cell(mapping_table, prim_lat, cell, symprec))
+      == NULL) {
+    warning_print("spglib: cel_trim_cell failed.");
+    warning_print(" (line %d, %s).\n", __LINE__, __FILE__);
+  }
 
   return primitive;
 
@@ -458,37 +492,95 @@ Cell * spa_transform_to_primitive(const Cell * cell,
   return NULL;
 }
 
-/* Return spacegroup.number = 0 if failed */
-static Spacegroup search_spacegroup(const Cell * primitive,
-                                    const int candidates[],
-                                    const int num_candidates,
-                                    const double symprec,
-                                    const double angle_tolerance)
+/* Return NULL if failed */
+Cell * spa_transform_from_primitive(const Cell * primitive,
+                                    const Centering centering,
+                                    const double symprec)
 {
-  Spacegroup spacegroup;
-  Symmetry *symmetry;
+  int multi, i, j, k, num_atom;
+  int *mapping_table;
+  double tmat[3][3], inv_tmat[3][3], shift[3][3];
+  Cell *std_cell, *trimmed_cell;
 
-  debug_print("search_spacegroup (tolerance = %f):\n", symprec);
+  mapping_table = NULL;
+  trimmed_cell = NULL;
+  std_cell = NULL;
 
-  symmetry = NULL;
-  spacegroup.number = 0;
-
-  if ((symmetry = sym_get_operation(primitive, symprec, angle_tolerance)) ==
-      NULL) {
+  switch (centering) {
+  case PRIMITIVE:
+    break;
+  case A_FACE:
+    mat_copy_matrix_d3(tmat, A_mat);
+    mat_inverse_matrix_d3(inv_tmat, A_mat, 0);
+    break;
+  case C_FACE:
+    mat_copy_matrix_d3(tmat, C_mat);
+    mat_inverse_matrix_d3(inv_tmat, C_mat, 0);
+    break;
+  case FACE:
+    mat_copy_matrix_d3(tmat, F_mat);
+    mat_inverse_matrix_d3(inv_tmat, F_mat, 0);
+    break;
+  case BODY:
+    mat_copy_matrix_d3(tmat, I_mat);
+    mat_inverse_matrix_d3(inv_tmat, I_mat, 0);
+    break;
+  case R_CENTER:
+    mat_copy_matrix_d3(tmat, R_mat);
+    mat_inverse_matrix_d3(inv_tmat, R_mat, 0);
+    break;
+  default:
     goto ret;
   }
 
-  spacegroup = search_spacegroup_with_symmetry(primitive,
-                                               candidates,
-                                               num_candidates,
-                                               symmetry,
-                                               symprec,
-                                               angle_tolerance);
-  sym_free_symmetry(symmetry);
-  symmetry = NULL;
+  multi = get_centering_shifts(shift, centering);
+
+  if ((mapping_table = (int*) malloc(sizeof(int) * primitive->size * multi))
+      == NULL) {
+    warning_print("spglib: Memory could not be allocated ");
+    goto ret;
+  }
+
+  if ((std_cell = cel_alloc_cell(primitive->size * multi)) == NULL) {
+    free(mapping_table);
+    mapping_table = NULL;
+    goto ret;
+  }
+
+  mat_multiply_matrix_d3(std_cell->lattice, primitive->lattice, inv_tmat);
+
+  num_atom = 0;
+  for (i = 0; i < primitive->size; i++) {
+    mat_multiply_matrix_vector_d3(std_cell->position[num_atom],
+                                  tmat,
+                                  primitive->position[i]);
+    std_cell->types[num_atom] = primitive->types[i];
+    num_atom++;
+  }
+
+  for (i = 0; i < multi - 1; i++) {
+    for (j = 0; j < primitive->size; j++) {
+      mat_copy_vector_d3(std_cell->position[num_atom],
+                         std_cell->position[j]);
+      for (k = 0; k < 3; k++) {
+        std_cell->position[num_atom][k] += shift[i][k];
+      }
+      std_cell->types[num_atom] = std_cell->types[j];
+      num_atom++;
+    }
+  }
+
+  trimmed_cell = cel_trim_cell(mapping_table,
+                               std_cell->lattice,
+                               std_cell,
+                               symprec);
+  cel_free_cell(std_cell);
+  std_cell = NULL;
+  free(mapping_table);
+  mapping_table = NULL;
 
  ret:
-  return spacegroup;
+  return trimmed_cell;
 }
 
 /* Return spacegroup.number = 0 if failed */
@@ -957,12 +1049,9 @@ static int match_hall_symbol_db(double origin_shift[3],
           hall_number == 452 ||
           hall_number == 458 ||
           hall_number == 460) {
-        mat_multiply_matrix_d3(changed_lattice,
-                               lattice,
-                               hR_to_hP);
-        if ((changed_symmetry = get_conventional_symmetry(hR_to_hP,
-                                                          R_CENTER,
-                                                          symmetry)) == NULL) {
+        mat_multiply_matrix_d3(changed_lattice, lattice, hR_to_hP);
+        if ((changed_symmetry =
+             get_conventional_symmetry(hR_to_hP, R_CENTER, symmetry)) == NULL) {
           goto err;
         }
 
@@ -978,7 +1067,17 @@ static int match_hall_symbol_db(double origin_shift[3],
           mat_copy_matrix_d3(lattice, changed_lattice);
           return 1;
         }
+      } else {
+        if (hal_match_hall_symbol_db(origin_shift,
+                                     lattice,
+                                     hall_number,
+                                     PRIMITIVE,
+                                     symmetry,
+                                     symprec)) {
+          return 1;
+        }
       }
+      break;
     }
     /* Do not break for other trigonal cases */
   default: /* HEXA, TETRA, TRICLI and rest of TRIGO */
@@ -1007,14 +1106,15 @@ static int match_hall_symbol_db_monocli(double origin_shift[3],
                                         const double symprec)
 {
   int i, j, k, l, is_found;
-  double vec[3], norms[3];
+  double vec[2][3], norms[3];
   Centering changed_centering;
   Symmetry * changed_symmetry;
   double changed_lattice[3][3];
 
   changed_symmetry = NULL;
 
-  for (i = 0; i < 18; i++) {
+  for (i = 0; i < 36; i++) {
+    /* centring type should be P or C */
     if (centering == C_FACE) {
       changed_centering = change_of_centering_monocli[i];
     } else { /* suppose PRIMITIVE */
@@ -1025,18 +1125,28 @@ static int match_hall_symbol_db_monocli(double origin_shift[3],
                            lattice,
                            change_of_basis_monocli[i]);
 
+    /* Make non-acute and length preference */
+    l = 0;
+    for (j = 0; j < 3; j++) {
+      if (j == change_of_unique_axis_monocli[i]) {continue;}
+      for (k = 0; k < 3; k++) {vec[l][k] = changed_lattice[k][j];}
+      norms[l] = mat_norm_squared_d3(vec[l]);
+      l++;
+    }
+
+    /* discard if principal angle is acute. */
+    if (vec[0][0] * vec[1][0] +
+        vec[0][1] * vec[1][1] +
+        vec[0][2] * vec[1][2] > 0) {
+      continue;
+    }
+
     /* Choose |a| < |b| < |c| if there are freedom. */
     if (num_hall_types == 3) {
-      l = 0;
-      for (j = 0; j < 3; j++) {
-        if (j == change_of_unique_axis_monocli[i]) {continue;}
-        for (k = 0; k < 3; k++) {vec[k] = changed_lattice[k][j];}
-        norms[l] = mat_norm_squared_d3(vec);
-        l++;
-      }
       if (norms[0] > norms[1]) {continue;}
     }
 
+
     if ((changed_symmetry =
          get_conventional_symmetry(change_of_basis_monocli[i],
                                    PRIMITIVE,
@@ -1147,7 +1257,7 @@ static Symmetry * get_conventional_symmetry(SPGCONST double transform_mat[3][3],
                                             const Symmetry *primitive_sym)
 {
   int i, j, k, multi, size;
-  double inv_tmat[3][3], shift[4][3];
+  double inv_tmat[3][3], shift[3][3];
   double symmetry_rot_d3[3][3], primitive_sym_rot_d3[3][3];
   Symmetry *symmetry;
 
@@ -1201,37 +1311,7 @@ static Symmetry * get_conventional_symmetry(SPGCONST double transform_mat[3][3],
   multi = 1;
 
   if (centering != PRIMITIVE) {
-    if (centering != FACE && centering != R_CENTER) {
-      for (i = 0; i < 3; i++) { shift[0][i] = 0.5; } /* BASE */
-      if (centering == A_FACE) { shift[0][0] = 0; }
-      if (centering == B_FACE) { shift[0][1] = 0; }
-      if (centering == C_FACE) { shift[0][2] = 0; }
-      multi = 2;
-    }
-
-    if (centering == R_CENTER) {
-      shift[0][0] = 2. / 3;
-      shift[0][1] = 1. / 3;
-      shift[0][2] = 1. / 3;
-      shift[1][0] = 1. / 3;
-      shift[1][1] = 2. / 3;
-      shift[1][2] = 2. / 3;
-      multi = 3;
-    }
-
-    if (centering == FACE) {
-      shift[0][0] = 0;
-      shift[0][1] = 0.5;
-      shift[0][2] = 0.5;
-      shift[1][0] = 0.5;
-      shift[1][1] = 0;
-      shift[1][2] = 0.5;
-      shift[2][0] = 0.5;
-      shift[2][1] = 0.5;
-      shift[2][2] = 0;
-      multi = 4;
-    }
-
+    multi = get_centering_shifts(shift, centering);
     for (i = 0; i < multi - 1; i++) {
       for (j = 0; j < size; j++) {
         mat_copy_matrix_i3(symmetry->rot[(i+1) * size + j],
@@ -1365,9 +1445,15 @@ static Centering get_base_center(SPGCONST int transform_mat[3][3])
   }
 
   /* body center */
-  if (abs(transform_mat[0][0]) +
-      abs(transform_mat[0][1]) +
-      abs(transform_mat[0][2]) == 2) {
+  if ((abs(transform_mat[0][0]) +
+       abs(transform_mat[0][1]) +
+       abs(transform_mat[0][2]) == 2) &&
+      (abs(transform_mat[1][0]) +
+       abs(transform_mat[1][1]) +
+       abs(transform_mat[1][2]) == 2) &&
+      (abs(transform_mat[2][0]) +
+       abs(transform_mat[2][1]) +
+       abs(transform_mat[2][2]) == 2)) {
     centering = BODY;
     goto end;
   }
@@ -1379,3 +1465,51 @@ static Centering get_base_center(SPGCONST int transform_mat[3][3])
  end:
   return centering;
 }
+
+static int get_centering_shifts(double shift[3][3],
+                                const Centering centering)
+{
+  int i, j, multi;
+
+  multi = 1;
+  for (i = 0; i < 3; i++) {
+    for (j = 0; j < 3; j++) {
+      shift[i][j] = 0;
+    }
+  }
+
+  if (centering != PRIMITIVE) {
+    if (centering != FACE && centering != R_CENTER) {
+      for (i = 0; i < 3; i++) { shift[0][i] = 0.5; } /* BASE */
+      if (centering == A_FACE) { shift[0][0] = 0; }
+      if (centering == B_FACE) { shift[0][1] = 0; }
+      if (centering == C_FACE) { shift[0][2] = 0; }
+      multi = 2;
+    }
+
+    if (centering == R_CENTER) {
+      shift[0][0] = 2. / 3;
+      shift[0][1] = 1. / 3;
+      shift[0][2] = 1. / 3;
+      shift[1][0] = 1. / 3;
+      shift[1][1] = 2. / 3;
+      shift[1][2] = 2. / 3;
+      multi = 3;
+    }
+
+    if (centering == FACE) {
+      shift[0][0] = 0;
+      shift[0][1] = 0.5;
+      shift[0][2] = 0.5;
+      shift[1][0] = 0.5;
+      shift[1][1] = 0;
+      shift[1][2] = 0.5;
+      shift[2][0] = 0.5;
+      shift[2][1] = 0.5;
+      shift[2][2] = 0;
+      multi = 4;
+    }
+  }
+
+  return multi;
+}
