@@ -39,6 +39,7 @@
 #include "cell.h"
 #include "debug.h"
 #include "delaunay.h"
+#include "determination.h"
 #include "kgrid.h"
 #include "kpoint.h"
 #include "mathfunc.h"
@@ -53,9 +54,6 @@
 #include "symmetry.h"
 #include "version.h"
 
-#define REDUCE_RATE 0.9
-#define NUM_ATTEMPT 10
-
 /*-------*/
 /* error */
 /*-------*/
@@ -89,12 +87,12 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
                                    const int hall_number,
                                    const double symprec,
                                    const double angle_tolerance);
+static SpglibDataset * init_dataset(void);
 static int set_dataset(SpglibDataset * dataset,
                        const Cell * cell,
-                       const Cell * primitive,
+                       const Primitive * primitive,
                        SPGCONST Spacegroup * spacegroup,
-                       const int * mapping_table,
-                       const double tolerance);
+                       ExactStructure *exstr);
 static int get_symmetry_from_dataset(int rotation[][3][3],
                                      double translation[][3],
                                      const int max_size,
@@ -114,13 +112,13 @@ static int get_symmetry_with_collinear_spin(int rotation[][3][3],
                                             const double spins[],
                                             const int num_atom,
                                             const double symprec,
-					    const double angle_tolerance);
+                                            const double angle_tolerance);
 static int get_multiplicity(SPGCONST double lattice[3][3],
                             SPGCONST double position[][3],
                             const int types[],
                             const int num_atom,
                             const double symprec,
-			    const double angle_tolerance);
+                            const double angle_tolerance);
 static int standardize_primitive(double lattice[3][3],
                                  double position[][3],
                                  int types[],
@@ -131,12 +129,14 @@ static int standardize_cell(double lattice[3][3],
                             double position[][3],
                             int types[],
                             const int num_atom,
+                            const int num_array_size,
                             const double symprec,
                             const double angle_tolerance);
 static int get_standardized_cell(double lattice[3][3],
                                  double position[][3],
                                  int types[],
                                  const int num_atom,
+                                 const int num_array_size,
                                  const int to_primitive,
                                  const double symprec,
                                  const double angle_tolerance);
@@ -151,7 +151,7 @@ static int get_international(char symbol[11],
                              const int types[],
                              const int num_atom,
                              const double symprec,
-			     const double angle_tolerance);
+                             const double angle_tolerance);
 static int get_schoenflies(char symbol[7],
                            SPGCONST double lattice[3][3],
                            SPGCONST double position[][3],
@@ -250,7 +250,7 @@ SpglibDataset * spg_get_dataset(SPGCONST double lattice[3][3],
                      num_atom,
                      0,
                      symprec,
-		     -1.0);
+                     -1.0);
 }
 
 /* Return NULL if failed */
@@ -267,7 +267,7 @@ SpglibDataset * spgat_get_dataset(SPGCONST double lattice[3][3],
                      num_atom,
                      0,
                      symprec,
-		     angle_tolerance);
+                     angle_tolerance);
 }
 
 /* Return NULL if failed */
@@ -284,7 +284,7 @@ SpglibDataset * spg_get_dataset_with_hall_number(SPGCONST double lattice[3][3],
                      num_atom,
                      hall_number,
                      symprec,
-		     -1.0);
+                     -1.0);
 }
 
 /* Return NULL if failed */
@@ -303,7 +303,7 @@ spgat_get_dataset_with_hall_number(SPGCONST double lattice[3][3],
                      num_atom,
                      hall_number,
                      symprec,
-		     angle_tolerance);
+                     angle_tolerance);
 }
 
 void spg_free_dataset(SpglibDataset *dataset)
@@ -321,6 +321,8 @@ void spg_free_dataset(SpglibDataset *dataset)
     dataset->wyckoffs = NULL;
     free(dataset->equivalent_atoms);
     dataset->equivalent_atoms = NULL;
+    free(dataset->mapping_to_primitive);
+    dataset->mapping_to_primitive = NULL;
     dataset->n_atoms = 0;
   }
 
@@ -329,6 +331,8 @@ void spg_free_dataset(SpglibDataset *dataset)
     dataset->std_positions = NULL;
     free(dataset->std_types);
     dataset->std_types = NULL;
+    free(dataset->std_mapping_to_primitive);
+    dataset->std_mapping_to_primitive = NULL;
     dataset->n_std_atoms = 0;
   }
 
@@ -339,7 +343,6 @@ void spg_free_dataset(SpglibDataset *dataset)
   strcpy(dataset->choice, "");
 
   free(dataset);
-  dataset = NULL;
 }
 
 /* Return 0 if failed */
@@ -360,7 +363,7 @@ int spg_get_symmetry(int rotation[][3][3],
                                    types,
                                    num_atom,
                                    symprec,
-				   -1.0);
+                                   -1.0);
 }
 
 /* Return 0 if failed */
@@ -382,7 +385,7 @@ int spgat_get_symmetry(int rotation[][3][3],
                                    types,
                                    num_atom,
                                    symprec,
-				   angle_tolerance);
+                                   angle_tolerance);
 }
 
 /* Return 0 if failed */
@@ -407,7 +410,7 @@ int spg_get_symmetry_with_collinear_spin(int rotation[][3][3],
                                           spins,
                                           num_atom,
                                           symprec,
-					  -1.0);
+                                          -1.0);
 }
 
 /* Return 0 if failed */
@@ -433,7 +436,7 @@ int spgat_get_symmetry_with_collinear_spin(int rotation[][3][3],
                                           spins,
                                           num_atom,
                                           symprec,
-					  angle_tolerance);
+                                          angle_tolerance);
 }
 
 int spg_get_hall_number_from_symmetry(SPGCONST int rotation[][3][3],
@@ -479,7 +482,7 @@ int spg_get_multiplicity(SPGCONST double lattice[3][3],
                           types,
                           num_atom,
                           symprec,
-			  -1.0);
+                          -1.0);
 }
 
 /* Return 0 if failed */
@@ -495,7 +498,7 @@ int spgat_get_multiplicity(SPGCONST double lattice[3][3],
                           types,
                           num_atom,
                           symprec,
-			  angle_tolerance);
+                          angle_tolerance);
 }
 
 /* Return 0 if failed */
@@ -512,7 +515,7 @@ int spg_get_international(char symbol[11],
                            types,
                            num_atom,
                            symprec,
-			   -1.0);
+                           -1.0);
 }
 
 /* Return 0 if failed */
@@ -530,7 +533,7 @@ int spgat_get_international(char symbol[11],
                            types,
                            num_atom,
                            symprec,
-			   angle_tolerance);
+                           angle_tolerance);
 }
 
 /* Return 0 if failed */
@@ -547,7 +550,7 @@ int spg_get_schoenflies(char symbol[7],
                          types,
                          num_atom,
                          symprec,
-			 -1.0);
+                         -1.0);
 }
 
 /* Return 0 if failed */
@@ -565,7 +568,7 @@ int spgat_get_schoenflies(char symbol[7],
                          types,
                          num_atom,
                          symprec,
-			 angle_tolerance);
+                         angle_tolerance);
 }
 
 /* Return 0 if failed */
@@ -643,7 +646,7 @@ SpglibSpacegroupType spg_get_spacegroup_type(const int hall_number)
   spglibtype.arithmetic_crystal_class_number = 0;
   strcpy(spglibtype.arithmetic_crystal_class_symbol, "");
 
-  if (0 < hall_number || hall_number < 531) {
+  if (0 < hall_number && hall_number < 531) {
     spgtype = spgdb_get_spacegroup_type(hall_number);
     spglibtype.number = spgtype.number;
     strcpy(spglibtype.schoenflies, spgtype.schoenflies);
@@ -701,16 +704,17 @@ int spgat_standardize_cell(double lattice[3][3],
                                    position,
                                    types,
                                    num_atom,
+                                   0,
                                    1,
                                    symprec,
-				   angle_tolerance);
+                                   angle_tolerance);
     } else {
       return standardize_primitive(lattice,
                                    position,
                                    types,
                                    num_atom,
                                    symprec,
-				   angle_tolerance);
+                                   angle_tolerance);
     }
   } else {
     if (no_idealize) {
@@ -719,15 +723,17 @@ int spgat_standardize_cell(double lattice[3][3],
                                    types,
                                    num_atom,
                                    0,
+                                   0,
                                    symprec,
-				   angle_tolerance);
+                                   angle_tolerance);
     } else {
       return standardize_cell(lattice,
                               position,
                               types,
                               num_atom,
+                              0,
                               symprec,
-			      angle_tolerance);
+                              angle_tolerance);
     }
   }
 }
@@ -744,7 +750,7 @@ int spg_find_primitive(double lattice[3][3],
                                types,
                                num_atom,
                                symprec,
-			       -1.0);
+                               -1.0);
 }
 
 /* Return 0 if failed */
@@ -760,7 +766,7 @@ int spgat_find_primitive(double lattice[3][3],
                                types,
                                num_atom,
                                symprec,
-			       angle_tolerance);
+                               angle_tolerance);
 }
 
 /* Return 0 if failed */
@@ -774,8 +780,9 @@ int spg_refine_cell(double lattice[3][3],
                           position,
                           types,
                           num_atom,
+                          0,
                           symprec,
-			  -1.0);
+                          -1.0);
 }
 
 /* Return 0 if failed */
@@ -790,8 +797,9 @@ int spgat_refine_cell(double lattice[3][3],
                           position,
                           types,
                           num_atom,
+                          0,
                           symprec,
-			  angle_tolerance);
+                          angle_tolerance);
 }
 
 int spg_delaunay_reduce(double lattice[3][3], const double symprec)
@@ -855,7 +863,7 @@ int spg_get_ir_reciprocal_mesh(int grid_address[][3],
                                 types,
                                 num_atom,
                                 symprec,
-				-1.0);
+                                -1.0);
 }
 
 int spg_get_stabilized_reciprocal_mesh(int grid_address[][3],
@@ -1006,44 +1014,18 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
                                    const double symprec,
                                    const double angle_tolerance)
 {
-  int attempt;
-  double tolerance;
-  Spacegroup spacegroup;
-  SpacegroupType spacegroup_type;
   SpglibDataset *dataset;
   Cell *cell;
-  Primitive *primitive;
+  DataContainer *container;
 
-  spacegroup.number = 0;
   dataset = NULL;
   cell = NULL;
-  primitive = NULL;
+  container = NULL;
 
-  if ((dataset = (SpglibDataset*) malloc(sizeof(SpglibDataset))) == NULL) {
-    warning_print("spglib: Memory could not be allocated.");
+  if ((dataset = init_dataset()) == NULL) {
     goto not_found;
   }
 
-  dataset->spacegroup_number = 0;
-  dataset->hall_number = 0;
-  strcpy(dataset->international_symbol, "");
-  strcpy(dataset->hall_symbol, "");
-  strcpy(dataset->choice, "");
-  dataset->origin_shift[0] = 0;
-  dataset->origin_shift[1] = 0;
-  dataset->origin_shift[2] = 0;
-  dataset->n_atoms = 0;
-  dataset->wyckoffs = NULL;
-  dataset->equivalent_atoms = NULL;
-  dataset->n_operations = 0;
-  dataset->rotations = NULL;
-  dataset->translations = NULL;
-  dataset->n_std_atoms = 0;
-  dataset->std_positions = NULL;
-  dataset->std_types = NULL;
-  /* dataset->pointgroup_number = 0; */
-  strcpy(dataset->pointgroup_symbol, "");
-
   if ((cell = cel_alloc_cell(num_atom)) == NULL) {
     free(dataset);
     dataset = NULL;
@@ -1059,59 +1041,20 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
     goto atoms_too_close;
   }
 
-  tolerance = symprec;
-  for (attempt = 0; attempt < NUM_ATTEMPT; attempt++) {
-    if ((primitive = spa_get_spacegroup(&spacegroup,
-					cell,
-					tolerance,
-					angle_tolerance))
-        == NULL) {
-      cel_free_cell(cell);
-      cell = NULL;
-      free(dataset);
-      dataset = NULL;
-      goto not_found;
+  if ((container = det_determine_all(cell,
+                                     hall_number,
+                                     symprec,
+                                     angle_tolerance))
+      != NULL) {
+    if (set_dataset(dataset,
+                    cell,
+                    container->primitive,
+                    container->spacegroup,
+                    container->exact_structure)) {
+      goto found;
     }
-
-    if (spacegroup.number > 0) {
-      /* With hall_number > 0, specific choice is searched. */
-      if (hall_number > 0) {
-        spacegroup_type = spgdb_get_spacegroup_type(hall_number);
-        if (spacegroup.number == spacegroup_type.number) {
-          spacegroup = spa_get_spacegroup_with_hall_number(primitive,
-                                                           hall_number);
-        } else {
-          prm_free_primitive(primitive);
-          primitive = NULL;
-          goto err;
-        }
-
-        if (spacegroup.number == 0) {
-          prm_free_primitive(primitive);
-          primitive = NULL;
-          goto err;
-        }
-      }
-
-      if (spacegroup.number > 0) {
-        if (set_dataset(dataset,
-                        cell,
-                        primitive->cell,
-                        &spacegroup,
-                        primitive->mapping_table,
-                        primitive->tolerance)) {
-          goto found;
-        } else {
-          tolerance *= REDUCE_RATE;
-        }
-      }
-    }
-
-    prm_free_primitive(primitive);
-    primitive = NULL;
   }
 
- err:
   cel_free_cell(cell);
   cell = NULL;
   free(dataset);
@@ -1126,8 +1069,7 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
   return NULL;
 
  found:
-  prm_free_primitive(primitive);
-  primitive = NULL;
+  det_free_container(container);
   cel_free_cell(cell);
   cell = NULL;
 
@@ -1135,23 +1077,53 @@ static SpglibDataset * get_dataset(SPGCONST double lattice[3][3],
   return dataset;
 }
 
+static SpglibDataset * init_dataset(void)
+{
+  SpglibDataset *dataset;
+
+  dataset = NULL;
+
+  if ((dataset = (SpglibDataset*) malloc(sizeof(SpglibDataset))) == NULL) {
+    warning_print("spglib: Memory could not be allocated.");
+    return NULL;
+  }
+
+  dataset->spacegroup_number = 0;
+  dataset->hall_number = 0;
+  strcpy(dataset->international_symbol, "");
+  strcpy(dataset->hall_symbol, "");
+  strcpy(dataset->choice, "");
+  dataset->origin_shift[0] = 0;
+  dataset->origin_shift[1] = 0;
+  dataset->origin_shift[2] = 0;
+  dataset->n_atoms = 0;
+  dataset->wyckoffs = NULL;
+  dataset->equivalent_atoms = NULL;
+  dataset->mapping_to_primitive = NULL;
+  dataset->n_operations = 0;
+  dataset->rotations = NULL;
+  dataset->translations = NULL;
+  dataset->n_std_atoms = 0;
+  dataset->std_positions = NULL;
+  dataset->std_types = NULL;
+  dataset->std_mapping_to_primitive = NULL;
+  /* dataset->pointgroup_number = 0; */
+  strcpy(dataset->pointgroup_symbol, "");
+
+  return dataset;
+}
+
 /* Return 0 if failed */
 static int set_dataset(SpglibDataset * dataset,
                        const Cell * cell,
-                       const Cell * primitive,
+                       const Primitive * primitive,
                        SPGCONST Spacegroup * spacegroup,
-                       const int * mapping_table,
-                       const double tolerance)
+                       ExactStructure *exstr)
 {
   int i;
   double inv_lat[3][3];
-  Cell *bravais;
-  Symmetry *symmetry;
   Pointgroup pointgroup;
 
-  bravais = NULL;
-  symmetry = NULL;
-
   /* Spacegroup type, transformation matrix, origin shift */
   dataset->n_atoms = cell->size;
   dataset->spacegroup_number = spacegroup->number;
@@ -1161,19 +1133,10 @@ static int set_dataset(SpglibDataset * dataset,
   strcpy(dataset->choice, spacegroup->choice);
   mat_inverse_matrix_d3(inv_lat, spacegroup->bravais_lattice, 0);
   mat_multiply_matrix_d3(dataset->transformation_matrix,
-                         inv_lat,
-                         cell->lattice);
+                         inv_lat, cell->lattice);
   mat_copy_vector_d3(dataset->origin_shift, spacegroup->origin_shift);
 
-  /* Symmetry operations */
-  if ((symmetry = ref_get_refined_symmetry_operations(cell,
-                                                      primitive,
-                                                      spacegroup,
-                                                      tolerance)) == NULL) {
-    return 0;
-  }
-
-  dataset->n_operations = symmetry->size;
+  dataset->n_operations = exstr->symmetry->size;
 
   if ((dataset->rotations =
        (int (*)[3][3]) malloc(sizeof(int[3][3]) * dataset->n_operations))
@@ -1189,9 +1152,9 @@ static int set_dataset(SpglibDataset * dataset,
     goto err;
   }
 
-  for (i = 0; i < symmetry->size; i++) {
-    mat_copy_matrix_i3(dataset->rotations[i], symmetry->rot[i]);
-    mat_copy_vector_d3(dataset->translations[i], symmetry->trans[i]);
+  for (i = 0; i < exstr->symmetry->size; i++) {
+    mat_copy_matrix_i3(dataset->rotations[i], exstr->symmetry->rot[i]);
+    mat_copy_vector_d3(dataset->translations[i], exstr->symmetry->trans[i]);
   }
 
   /* Wyckoff positions */
@@ -1202,25 +1165,41 @@ static int set_dataset(SpglibDataset * dataset,
   }
 
   if ((dataset->equivalent_atoms =
-       (int*) malloc(sizeof(int) * dataset->n_atoms))
-      == NULL) {
+       (int*) malloc(sizeof(int) * dataset->n_atoms)) == NULL) {
     warning_print("spglib: Memory could not be allocated.");
     goto err;
   }
 
-  if ((bravais = ref_get_Wyckoff_positions(dataset->wyckoffs,
-                                           dataset->equivalent_atoms,
-                                           primitive,
-                                           cell,
-                                           spacegroup,
-                                           symmetry,
-                                           mapping_table,
-                                           tolerance)) == NULL) {
+  for (i = 0; i < dataset->n_atoms; i++) {
+    dataset->wyckoffs[i] = exstr->wyckoffs[i];
+    dataset->equivalent_atoms[i] = exstr->equivalent_atoms[i];
+  }
+
+  if ((dataset->mapping_to_primitive =
+       (int*) malloc(sizeof(int) * dataset->n_atoms)) == NULL) {
+    warning_print("spglib: Memory could not be allocated.");
     goto err;
   }
 
-  dataset->n_std_atoms = bravais->size;
-  mat_copy_matrix_d3(dataset->std_lattice, bravais->lattice);
+  debug_print("Refined cell after ref_get_Wyckoff_positions\n");
+  debug_print(" (line %d, %s).\n", __LINE__, __FILE__);
+  debug_print_matrix_d3(exstr->bravais->lattice);
+#ifdef SPGDEBUG
+  for (i = 0; i < bravais->size; i++) {
+    printf("%d: %f %f %f\n",
+           exstr->bravais->types[i],
+           exstr->bravais->position[i][0],
+           exstr->bravais->position[i][1],
+           exstr->bravais->position[i][2]);
+  }
+#endif
+
+  for (i = 0; i < dataset->n_atoms; i++) {
+    dataset->mapping_to_primitive[i] = primitive->mapping_table[i];
+  }
+
+  dataset->n_std_atoms = exstr->bravais->size;
+  mat_copy_matrix_d3(dataset->std_lattice, exstr->bravais->lattice);
 
   if ((dataset->std_positions =
        (double (*)[3]) malloc(sizeof(double[3]) * dataset->n_std_atoms))
@@ -1235,15 +1214,17 @@ static int set_dataset(SpglibDataset * dataset,
     goto err;
   }
 
-  for (i = 0; i < dataset->n_std_atoms; i++) {
-    mat_copy_vector_d3(dataset->std_positions[i], bravais->position[i]);
-    dataset->std_types[i] = bravais->types[i];
+  if ((dataset->std_mapping_to_primitive =
+       (int*) malloc(sizeof(int) * dataset->n_std_atoms)) == NULL) {
+    warning_print("spglib: Memory could not be allocated.");
+    goto err;
   }
 
-  cel_free_cell(bravais);
-  bravais = NULL;
-  sym_free_symmetry(symmetry);
-  symmetry = NULL;
+  for (i = 0; i < dataset->n_std_atoms; i++) {
+    mat_copy_vector_d3(dataset->std_positions[i], exstr->bravais->position[i]);
+    dataset->std_types[i] = exstr->bravais->types[i];
+    dataset->std_mapping_to_primitive[i] = exstr->std_mapping_to_primitive[i];
+  }
 
   /* dataset->pointgroup_number = spacegroup->pointgroup_number; */
   pointgroup = ptg_get_pointgroup(spacegroup->pointgroup_number);
@@ -1256,14 +1237,18 @@ static int set_dataset(SpglibDataset * dataset,
     free(dataset->std_positions);
     dataset->std_positions = NULL;
   }
-  if (bravais != NULL) {
-    cel_free_cell(bravais);
-    bravais = NULL;
+  if (dataset->std_mapping_to_primitive != NULL) {
+    free(dataset->std_mapping_to_primitive);
+    dataset->std_mapping_to_primitive = NULL;
   }
   if (dataset->equivalent_atoms != NULL) {
     free(dataset->equivalent_atoms);
     dataset->equivalent_atoms = NULL;
   }
+  if (dataset->mapping_to_primitive != NULL) {
+    free(dataset->mapping_to_primitive);
+    dataset->mapping_to_primitive = NULL;
+  }
   if (dataset->wyckoffs != NULL) {
     free(dataset->wyckoffs);
     dataset->wyckoffs = NULL;
@@ -1276,10 +1261,6 @@ static int set_dataset(SpglibDataset * dataset,
     free(dataset->rotations);
     dataset->rotations = NULL;
   }
-  if (symmetry != NULL) {
-    sym_free_symmetry(symmetry);
-    symmetry = NULL;
-  }
 
   return 0;
 }
@@ -1307,7 +1288,7 @@ static int get_symmetry_from_dataset(int rotation[][3][3],
                              num_atom,
                              0,
                              symprec,
-			     angle_tolerance)) == NULL) {
+                             angle_tolerance)) == NULL) {
     return 0;
   }
 
@@ -1326,10 +1307,12 @@ static int get_symmetry_from_dataset(int rotation[][3][3],
   }
 
   spg_free_dataset(dataset);
+  dataset = NULL;
   return num_sym;
 
  err:
   spg_free_dataset(dataset);
+  dataset = NULL;
   spglib_error_code = SPGERR_ARRAY_SIZE_SHORTAGE;
   return 0;
 }
@@ -1370,7 +1353,7 @@ static int get_symmetry_with_collinear_spin(int rotation[][3][3],
                              num_atom,
                              0,
                              symprec,
-			     angle_tolerance)) == NULL) {
+                             angle_tolerance)) == NULL) {
     cel_free_cell(cell);
     cell = NULL;
     goto get_dataset_failed;
@@ -1378,6 +1361,7 @@ static int get_symmetry_with_collinear_spin(int rotation[][3][3],
 
   if ((sym_nonspin = sym_alloc_symmetry(dataset->n_operations)) == NULL) {
     spg_free_dataset(dataset);
+    dataset = NULL;
     cel_free_cell(cell);
     cell = NULL;
     goto err;
@@ -1388,6 +1372,7 @@ static int get_symmetry_with_collinear_spin(int rotation[][3][3],
     mat_copy_vector_d3(sym_nonspin->trans[i], dataset->translations[i]);
   }
   spg_free_dataset(dataset);
+  dataset = NULL;
 
   if ((symmetry = spn_get_collinear_operations(equivalent_atoms,
                                                sym_nonspin,
@@ -1454,12 +1439,13 @@ static int get_multiplicity(SPGCONST double lattice[3][3],
                              num_atom,
                              0,
                              symprec,
-			     angle_tolerance)) == NULL) {
+                             angle_tolerance)) == NULL) {
     return 0;
   }
 
   size = dataset->n_operations;
   spg_free_dataset(dataset);
+  dataset = NULL;
 
   return size;
 }
@@ -1471,7 +1457,8 @@ static int standardize_primitive(double lattice[3][3],
                                  const double symprec,
                                  const double angle_tolerance)
 {
-  int num_prim_atom;
+  int i, num_prim_atom;
+  int *mapping_table;
   Centering centering;
   SpglibDataset *dataset;
   Cell *primitive, *bravais;
@@ -1481,6 +1468,7 @@ static int standardize_primitive(double lattice[3][3],
                            { 0, 0, 1 }};
 
   num_prim_atom = 0;
+  mapping_table = NULL;
   dataset = NULL;
   primitive = NULL;
   bravais = NULL;
@@ -1491,17 +1479,20 @@ static int standardize_primitive(double lattice[3][3],
                              num_atom,
                              0,
                              symprec,
-			     angle_tolerance)) == NULL) {
+                             angle_tolerance)) == NULL) {
     return 0;
   }
 
   if ((centering = get_centering(dataset->hall_number)) == CENTERING_ERROR) {
+    spg_free_dataset(dataset);
+    dataset = NULL;
     goto err;
   }
 
   if ((bravais = cel_alloc_cell(dataset->n_std_atoms)) == NULL) {
     spg_free_dataset(dataset);
-    return 0;
+    dataset = NULL;
+    goto err;
   }
 
   cel_set_cell(bravais,
@@ -1510,11 +1501,38 @@ static int standardize_primitive(double lattice[3][3],
                dataset->std_types);
 
   spg_free_dataset(dataset);
+  dataset = NULL;
 
-  primitive = spa_transform_to_primitive(bravais,
+  if ((mapping_table = (int*) malloc(sizeof(int) * bravais->size)) == NULL) {
+    warning_print("spglib: Memory could not be allocated ");
+    cel_free_cell(bravais);
+    bravais = NULL;
+    goto err;
+  }
+
+  primitive = spa_transform_to_primitive(mapping_table,
+                                         bravais,
                                          identity,
                                          centering,
                                          symprec);
+
+  for (i = 0; i < primitive->size; i++) {
+    /* This is an assertion. */
+    if (mapping_table[i] != i) {
+      warning_print("spglib: spa_transform_to_primitive failed.");
+      warning_print("Unexpected atom index mapping to primitive (%d != %d).\n",
+                    mapping_table[i], i);
+      warning_print(" (line %d, %s).\n", __LINE__, __FILE__);
+      free(mapping_table);
+      mapping_table = NULL;
+      cel_free_cell(bravais);
+      bravais = NULL;
+      goto err;
+    }
+  }
+
+  free(mapping_table);
+  mapping_table = NULL;
   cel_free_cell(bravais);
   bravais = NULL;
 
@@ -1539,6 +1557,7 @@ static int standardize_cell(double lattice[3][3],
                             double position[][3],
                             int types[],
                             const int num_atom,
+                            const int num_array_size,
                             const double symprec,
                             const double angle_tolerance)
 {
@@ -1554,12 +1573,17 @@ static int standardize_cell(double lattice[3][3],
                              num_atom,
                              0,
                              symprec,
-			     angle_tolerance)) == NULL) {
-    return 0;
+                             angle_tolerance)) == NULL) {
+    goto err;
+  }
+
+  if (num_array_size > 0) {
+    if (num_atom < dataset->n_std_atoms) {
+      goto array_size_shortage_err;
+    }
   }
 
   n_std_atoms = dataset->n_std_atoms;
-
   mat_copy_matrix_d3(lattice, dataset->std_lattice);
   for (i = 0; i < dataset->n_std_atoms; i++) {
     types[i] = dataset->std_types[i];
@@ -1567,27 +1591,40 @@ static int standardize_cell(double lattice[3][3],
   }
 
   spg_free_dataset(dataset);
+  dataset = NULL;
 
   return n_std_atoms;
+
+err:
+  spglib_error_code = SPGERR_CELL_STANDARDIZATION_FAILED;
+  return 0;
+
+array_size_shortage_err:
+  spglib_error_code = SPGERR_ARRAY_SIZE_SHORTAGE;
+  return 0;
 }
 
 static int get_standardized_cell(double lattice[3][3],
                                  double position[][3],
                                  int types[],
                                  const int num_atom,
+                                 const int num_array_size,
                                  const int to_primitive,
                                  const double symprec,
                                  const double angle_tolerance)
 {
-  int num_std_atom;
+  int i, num_std_atom, num_prim_atom;
+  int *mapping_table;
   SpglibDataset *dataset;
-  Cell *std_cell, *cell;
+  Cell *std_cell, *cell, *primitive;
   Centering centering;
 
   num_std_atom = 0;
+  mapping_table = NULL;
   dataset = NULL;
   std_cell = NULL;
   cell = NULL;
+  primitive = NULL;
 
   if ((dataset = get_dataset(lattice,
                              position,
@@ -1595,47 +1632,109 @@ static int get_standardized_cell(double lattice[3][3],
                              num_atom,
                              0,
                              symprec,
-			     angle_tolerance)) == NULL) {
+                             angle_tolerance)) == NULL) {
     goto err;
   }
 
-  if (to_primitive) {
-    if ((centering = get_centering(dataset->hall_number)) == CENTERING_ERROR) {
-      goto err;
-    }
-  } else {
-    centering = PRIMITIVE;
+  if ((centering = get_centering(dataset->hall_number)) == CENTERING_ERROR) {
+    goto err;
   }
 
   if ((cell = cel_alloc_cell(num_atom)) == NULL) {
     spg_free_dataset(dataset);
+    dataset = NULL;
     goto err;
   }
 
   cel_set_cell(cell, lattice, position, types);
-  std_cell = spa_transform_to_primitive(cell,
-                                        dataset->transformation_matrix,
-                                        centering,
-                                        symprec);
-  spg_free_dataset(dataset);
+
+  if ((mapping_table = (int*) malloc(sizeof(int) * cell->size)) == NULL) {
+    warning_print("spglib: Memory could not be allocated ");
+    cel_free_cell(cell);
+    cell = NULL;
+    spg_free_dataset(dataset);
+    dataset = NULL;
+    goto err;
+  }
+
+  if ((primitive = spa_transform_to_primitive(mapping_table,
+                                              cell,
+                                              dataset->transformation_matrix,
+                                              centering,
+                                              symprec)) == NULL) {
+    warning_print("spglib: spa_transform_to_primitive failed.");
+    warning_print(" (line %d, %s).\n", __LINE__, __FILE__);
+  }
+
+  for (i = 0; i < cell->size; i++) {
+    /* This is an assertion. */
+    if (mapping_table[i] != dataset->mapping_to_primitive[i]) {
+      warning_print("spglib: spa_transform_to_primitive failed.");
+      warning_print("Unexpected atom index mapping to primitive (%d != %d).\n",
+                    mapping_table[i], dataset->mapping_to_primitive[i]);
+      warning_print(" (line %d, %s).\n", __LINE__, __FILE__);
+      free(mapping_table);
+      mapping_table = NULL;
+      cel_free_cell(cell);
+      cell = NULL;
+      spg_free_dataset(dataset);
+      dataset = NULL;
+      goto err;
+    }
+  }
+
+  free(mapping_table);
+  mapping_table = NULL;
   cel_free_cell(cell);
   cell = NULL;
+  spg_free_dataset(dataset);
+  dataset = NULL;
+
+  if (primitive == NULL) {
+    goto err;
+  }
+
+  if (to_primitive || centering == PRIMITIVE) {
+    set_cell(lattice, position, types, primitive);
+    num_prim_atom = primitive->size;
+    cel_free_cell(primitive);
+    primitive = NULL;
+    return num_prim_atom;
+  }
+
+  if ((std_cell = spa_transform_from_primitive(primitive, centering, symprec))
+      == NULL) {
+    warning_print("spglib: spa_transform_from_primitive failed.");
+    warning_print(" (line %d, %s).\n", __LINE__, __FILE__);
+  }
+  cel_free_cell(primitive);
+  primitive = NULL;
 
   if (std_cell == NULL) {
     goto err;
   }
 
-  set_cell(lattice, position, types, std_cell);
-  num_std_atom = std_cell->size;
+  if (num_array_size > 0) {
+    if (num_array_size < std_cell->size) {
+      cel_free_cell(std_cell);
+      std_cell = NULL;
+      goto array_size_shortage_err;
+    }
+  }
 
+  num_std_atom = std_cell->size;
+  set_cell(lattice, position, types, std_cell);
   cel_free_cell(std_cell);
   std_cell = NULL;
-
   return num_std_atom;
 
- err:
+err:
   spglib_error_code = SPGERR_CELL_STANDARDIZATION_FAILED;
   return 0;
+
+array_size_shortage_err:
+  spglib_error_code = SPGERR_ARRAY_SIZE_SHORTAGE;
+  return 0;
 }
 
 static void set_cell(double lattice[3][3],
@@ -1667,44 +1766,36 @@ static int get_international(char symbol[11],
                              const int types[],
                              const int num_atom,
                              const double symprec,
-			     const double angle_tolerance)
+                             const double angle_tolerance)
 {
-  Cell *cell;
-  Primitive *primitive;
-  Spacegroup spacegroup;
+  SpglibDataset *dataset;
+  int number;
 
-  cell = NULL;
-  primitive = NULL;
-  spacegroup.number = 0;
+  dataset = NULL;
 
-  if ((cell = cel_alloc_cell(num_atom)) == NULL) {
+  if ((dataset = get_dataset(lattice,
+                             position,
+                             types,
+                             num_atom,
+                             0,
+                             symprec,
+                             angle_tolerance)) == NULL) {
     goto err;
   }
 
-  cel_set_cell(cell, lattice, position, types);
-
-  if ((primitive = spa_get_spacegroup(&spacegroup,
-				      cell,
-				      symprec,
-				      angle_tolerance)) == NULL) {
-    cel_free_cell(cell);
-    cell = NULL;
-    goto err;
-  }
-
-  prm_free_primitive(primitive);
-  primitive = NULL;
-  cel_free_cell(cell);
-  cell = NULL;
-
-  if (spacegroup.number > 0) {
-    strcpy(symbol, spacegroup.international_short);
+  if (dataset->spacegroup_number > 0) {
+    number = dataset->spacegroup_number;
+    strcpy(symbol, dataset->international_symbol);
+    spg_free_dataset(dataset);
+    dataset = NULL;
   } else {
+    spg_free_dataset(dataset);
+    dataset = NULL;
     goto err;
   }
 
   spglib_error_code = SPGLIB_SUCCESS;
-  return spacegroup.number;
+  return number;
 
  err:
   spglib_error_code = SPGERR_SPACEGROUP_SEARCH_FAILED;
@@ -1719,47 +1810,40 @@ static int get_schoenflies(char symbol[7],
                            const double symprec,
                            const double angle_tolerance)
 {
-  Cell *cell;
-  Primitive *primitive;
-  Spacegroup spacegroup;
+  SpglibDataset *dataset;
+  SpglibSpacegroupType spgtype;
+  int number;
 
-  cell = NULL;
-  primitive = NULL;
-  spacegroup.number = 0;
+  dataset = NULL;
 
-  if ((cell = cel_alloc_cell(num_atom)) == NULL) {
+  if ((dataset = get_dataset(lattice,
+                             position,
+                             types,
+                             num_atom,
+                             0,
+                             symprec,
+                             angle_tolerance)) == NULL) {
     goto err;
   }
 
-  cel_set_cell(cell, lattice, position, types);
-
-  if ((primitive = spa_get_spacegroup(&spacegroup,
-				      cell,
-				      symprec,
-				      angle_tolerance)) == NULL) {
-    cel_free_cell(cell);
-    cell = NULL;
-    goto err;
-  }
-
-  prm_free_primitive(primitive);
-  primitive = NULL;
-  cel_free_cell(cell);
-  cell = NULL;
-
-  if (spacegroup.number > 0) {
-    strcpy(symbol, spacegroup.schoenflies);
+  if (dataset->spacegroup_number > 0) {
+    number = dataset->spacegroup_number;
+    spgtype = spg_get_spacegroup_type(dataset->hall_number);
+    strcpy(symbol, spgtype.schoenflies);
+    spg_free_dataset(dataset);
+    dataset = NULL;
   } else {
+    spg_free_dataset(dataset);
+    dataset = NULL;
     goto err;
   }
 
   spglib_error_code = SPGLIB_SUCCESS;
-  return spacegroup.number;
+  return number;
 
  err:
   spglib_error_code = SPGERR_SPACEGROUP_SEARCH_FAILED;
   return 0;
-
 }
 
 
@@ -1788,12 +1872,13 @@ static int get_ir_reciprocal_mesh(int grid_address[][3],
                              num_atom,
                              0,
                              symprec,
-			     angle_tolerance)) == NULL) {
+                             angle_tolerance)) == NULL) {
     return 0;
   }
 
   if ((rotations = mat_alloc_MatINT(dataset->n_operations)) == NULL) {
     spg_free_dataset(dataset);
+    dataset = NULL;
     return 0;
   }
 
@@ -1811,6 +1896,7 @@ static int get_ir_reciprocal_mesh(int grid_address[][3],
   mat_free_MatINT(rotations);
   rotations = NULL;
   spg_free_dataset(dataset);
+  dataset = NULL;
   return num_ir;
 }
 
